use crate::utils;

#[derive(Debug,Clone)]
struct File {
    name: String,
    size: i32,
}

#[derive(Debug,Clone)]
struct Folder{
    root: Option<Box<Folder>>,
    name: String,
    folder: Vec<Folder>,
    files: Vec<File>,
}

impl Folder {
    fn add_folder(&mut self, folder: Folder) {
        self.folder.push(folder);
    }

    fn add_file(&mut self, file: File) {
        self.files.push(file);
    }

    fn get_size(self) -> i32 {
        let file_sum: i32 = self.files.into_iter().map(|file| file.size).sum();
        let folder_sum: i32 = self.folder.into_iter().map(|folder| folder.get_size()).sum();

        return file_sum + folder_sum;
    }

    fn flat_folder(self) -> Vec<Folder> {
        let mut sub_folder = vec![];
        for folder in self.folder {
            sub_folder.push(folder);
            sub_folder.extend(folder.flat_folder());
        }

        return sub_folder;
    }
}


pub fn execute_task01(content: &str) {
    let line = solve_task01(content);

    println!("Day07 - Task01 - Sum of total Size: {}", line);
}

pub fn execute_task02(content: &str) {
    let line = solve_task02(content);

    println!("Day07 - Task02 - Last Item Str Line: {}", line);
}

fn solve_task01(content: &str) -> i32 {

    let root: &Folder = parse_tree(content);

    let full_size: i32 = root.flat_folder()
        .into_iter()
        .map(|folder| folder.get_size())
        .filter(|size| size.clone() <= 100000)
        .sum();

    return full_size;
}

fn solve_task02(content: &str) -> i32 {
    return 0;
}

fn parse_tree(content: &str) -> &Folder {
    let vector = utils::convert_to_string_slice(&content);

    let root: &Folder = &Folder{
        root: None,
        name: "".parse().unwrap(),
        folder: vec![],
        files: vec![],
    };
    let mut current: &Folder = &root;

    for line in vector.into_iter() {
        if line.starts_with("$") {
            if line.starts_with("$ cd") {
                let select = line.split(" ").collect::<Vec<&str>>()[3..].join("");

                if select == "/" {
                    current = &root;
                } else if select == ".." {
                    current = &current.root.unwrap();
                } else {
                    for child_folder in current.folder {
                        if child_folder.name == select {
                            current = &child_folder;
                            break;
                        }
                    }
                }
            } else if line.starts_with("$ ls") {
                continue
            } else {
                let parts: Vec<&str> = line.split(" ").collect();
                let part1 = *parts.get(0).unwrap();
                let part2 = *parts.get(1).unwrap();

                if part1 == "dir" {
                    current.add_folder(Folder {
                        root: Some(Box::new(*current)),
                        name: part2.parse().unwrap(),
                        folder: vec![],
                        files: vec![],
                    });
                } else {
                    current.add_file(File{
                        name: part2.parse().unwrap(),
                        size: part1.parse::<i32>().unwrap(),
                    });
                }
            }
        }
    }

    return root;
}

#[cfg(test)]
mod test {

    const EXAMPLE01: &'static str = include_str!("input/day07/example01");

    #[test]
    fn test_first() {
        assert_eq!(super::solve_task01(EXAMPLE01), 95437);

    }

    #[test]
    fn test_second() {

    }
}